__title__ = "bandits"
__description__ = "Cythonized bandit algorithms"
__uri__ = "https://github.com/jbn/bandits"
__doc__ = __description__ + " <" + __uri__ + ">"
__license__ = " BSD-3-Clause"
__copyright__ = "Copyright (c) 2017 John Bjorn Nelson"
__version__ = "0.0.1"
__author__ = "John Bjorn Nelson"
__email__ = "jbn@abreka.com"
